/*
 * arch/arm/common/wbi.h
 *
 * WBI header
 *
 * Copyright 2011 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */

#ifndef __WBI_H__
#define __WBI_H__

#define RU_SIZE(len, alignment) (((len) + (alignment) -1)&~((alignment)-1))
#define RU_SECTOR_SIZE(len) RU_SIZE(len, wb_default_device->phys_sector_size)
#define INIT_CKSUM(cksum) do{(cksum) = 0x0F0F0F0F;}while(0)
#define wb_msg(fmt,arg...)                                   \
        do{                                                    \
	        if(!(wb_default_device->flag&WBI_SILENT_MODE)) \
                        printk(fmt, ##arg);                    \
	}while(0);
extern struct wb_device *wb_default_device;
extern struct wbi_compressor *wbi_default_compressor;
extern struct wb_header wbheader;

extern int wb_printk(const char *fmt, ...);
extern int wbi_is_canceled(void);
extern int wbi_is_timeout(void);
extern int wbi_section_idx(struct wb_section *p);
extern void wbi_calc_cksum(ulong *cksum, ulong *buf, ulong size);
extern void wbi_recalc_cksum(ulong *cksum, ulong *buf, ulong size);

/* WBI compress sequencer API */
extern int wbi_setup(ulong work, ulong *wlen, ulong dst, ulong *dlen);
extern int wbi_send_data(void *b, ulong len);
extern int wbi_send_sections(struct wb_header *header, struct wb_section *s, ulong n);
extern void wbi_align_sector(void);
extern int wbi_flush(void);
extern int wbi_rewrite_header(struct wb_header *header);
extern void wbi_stat(void);

#endif /* __WBI_H__ */
