/*
 * arch/arm/mach-cxd4132/cachectrl.c
 *
 * cache lockdown driver
 *
 * Copyright 2011 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/smp.h>
#include <linux/seq_file.h>
#include <linux/io.h>
#include <asm/hardware/cache-pl310.h>

#ifdef CONFIG_CACHE_PL310

#define N_MASTER 3

void l2cachectrl_write(char *arg, size_t len)
{
	int n, i;
	uint mask[N_MASTER];

	n = sscanf(arg, "%x %x %x", &mask[0], &mask[1], &mask[2]);
	if (n <= 0) {
		printk(KERN_ERR "invalid format\n");
		return;
	}
	for (i = n; i < 3; i++) {
		mask[i] = mask[n - 1];
	}
	pl310_lockdown(mask, N_MASTER);
}

int l2cachectrl_show(struct seq_file *seq, void *v)
{
	int i;
	__u32 lock_d, lock_i;

	seq_printf(seq, "L2:");
	for (i = 0; i < N_MASTER; i++) {
		lock_d = readl(PL310_REG(PL310_LOCKDOWN_WAY_D(i)));
		lock_i = readl(PL310_REG(PL310_LOCKDOWN_WAY_I(i)));
		seq_printf(seq, " 0x%02x,0x%02x", lock_d, lock_i);
	}
	seq_printf(seq, "\n");
	return 0;
}
#endif /* CONFIG_CACHE_PL310 */
