/*
 * arch/arm/mach-cxd4132/include/mach/vectors-layout.h
 *
 * Exception vector page layout
 *
 * Copyright 2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#define FIQ_VEC_OFFS		0x0000001c
#define FIQ_VEC_END		0x00000200
#define MAX_FIQ_VEC		(FIQ_VEC_END - FIQ_VEC_OFFS)

#define CXD4132_FIQ_REGSAVE	0x00000600
# define CXD4132_FIQ_N_REGS	24
# define CXD4132_FIQ_REGSIZE	(4 * CXD4132_FIQ_N_REGS)
# define CXD4132_MAX_CPUS	3
# define CXD4132_FIQ_REG_ALL	(CXD4132_FIQ_REGSIZE * CXD4132_MAX_CPUS)

#define CXD4132_FIQ_DEBUG	0x00000800
#define CXD4132_FIQ_DEBUG_END	0x00000f00
#define MAX_FIQ_DEBUG		(CXD4132_FIQ_DEBUG_END - CXD4132_FIQ_DEBUG)

#define VA_VECTORS_REGSAVE	(CONFIG_VECTORS_BASE + CXD4132_FIQ_REGSAVE)
#define VA_VECTORS_DEBUG	(CONFIG_VECTORS_BASE + CXD4132_FIQ_DEBUG)

#if CXD4132_FIQ_REGSAVE + CXD4132_FIQ_REG_ALL >= CXD4132_FIQ_DEBUG
# error "CXD4132_FIQ_N_REGS too big"
#endif

#ifndef __ASSEMBLY__
extern char fiq_vec[], fiq_vec_end[];
extern char fiq_debug[], fiq_debug_end[];
extern void mach_trap_init(unsigned long);
extern void cxd4132_fiq_init(void);
#endif /* !__ASSEMBLY__ */

#ifdef VMLINUX_SYMBOL
/* include from vmlinux.lds.S */
ASSERT((fiq_vec_end - fiq_vec <= MAX_FIQ_VEC), "FIQ VECTOR too big")
ASSERT((fiq_debug_end - fiq_debug <= MAX_FIQ_DEBUG), "FIQ DEBUG too big")
#endif
