/*
 * arch/arm/plat-cxd41xx/clock.c
 *
 *
 * Copyright 2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include <linux/module.h>
#include <linux/udif/types.h>
#include <linux/udif/macros.h>
#include <linux/udif/device.h>
#include <linux/udif/spinlock.h>

static UDIF_DECLARE_SPINLOCK(udif_clock_lock);

UDIF_ERR __udif_devio_hclk(UDIF_CLOCK *clock, UDIF_U8 enable)
{
	UDIF_PARM_CHK(!clock, "invalid clock", UDIF_ERR_PAR);
	UDIF_PARM_CHK(!clock->hclk.set, "invalid hclk", UDIF_ERR_PAR);
	if (enable >= 2) {
		return UDIF_ERR_PAR;
	}

	if (enable) {
		udif_iowrite32(1<<clock->hclk.shift, clock->hclk.set);
	}
	else {
		udif_iowrite32(1<<clock->hclk.shift, clock->hclk.clr);
	}
	return UDIF_ERR_OK;
}


UDIF_ERR __udif_devio_devclk(UDIF_CLOCK *clock, UDIF_U8 enable, UDIF_U8 clksel)
{
	UDIF_U32 tmp, flags;

	UDIF_PARM_CHK(!clock, "invalid clock", UDIF_ERR_PAR);
	UDIF_PARM_CHK(!clock->dclk.set, "invalid devclk", UDIF_ERR_PAR);
	UDIF_PARM_CHK(!clock->dclk.clr, "invalid devclk", UDIF_ERR_PAR);
	if (enable >= 2) {
		return UDIF_ERR_PAR;
	}

	if (enable) {
		if ((clock->dclksel.set) && !(clock->dclksel.clr)) {
			udif_spin_lock_irqsave(&udif_clock_lock, flags);
			tmp = udif_ioread32(clock->dclksel.set);
			tmp &= ~(0xf<<clock->dclksel.shift);
			tmp |= clksel<<clock->dclksel.shift;
			udif_iowrite32(tmp, clock->dclksel.set);
			udif_spin_unlock_irqrestore(&udif_clock_lock, flags);
		}
		else {
			if (clock->dclksel.clr) {
				udif_iowrite32(0xf<<clock->dclksel.shift,
					       clock->dclksel.clr);
			}
			if (clock->dclksel.set) {
				udif_iowrite32(clksel<<clock->dclksel.shift,
					       clock->dclksel.set);
			}
		}
		udif_iowrite32(1<<clock->dclk.shift, clock->dclk.set);
	}
	else {
		udif_iowrite32(1<<clock->dclk.shift, clock->dclk.clr);
	}
	return UDIF_ERR_OK;
}


EXPORT_SYMBOL(__udif_devio_hclk);
EXPORT_SYMBOL(__udif_devio_devclk);
