/*
 * arch/arm/plat-cxd41xx/error_remote.c
 *
 * Remote error
 *
 * Copyright 2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <mach/moduleparam.h>
#include <linux/init.h>
#include <linux/cpu.h>
#include <linux/interrupt.h>
#include <mach/errind.h>
#include <mach/errind_remote.h>
#include <mach/cxd41xx_cfg.h>
#include <asm/io.h>
#include <mach/hardware.h>
#include <mach/regs-gpio.h>

extern int cxd41xx_smode;

void errind_remote(int code)
{
	unsigned int port, bit;

	if (cxd41xx_smode < 0) {
#if 0
		/* local loopback */
		errind_start(code);
#endif
		return;
	}
	if (!cxd41xx_smode)
		return;
	if (CXD41XX_PORT_UNDEF == cxd41xx_abt1_port)
		return;
	port = cxd41xx_abt1_port & 0xff;
	bit  = (cxd41xx_abt1_port >> 8) & 0xff;
	writel(1 << bit, VA_GPIO(port)+GPIO_DATA_CLEAR);
}

#define ABT1_NAME "abt1_intr"
static int abt1_info;

static irqreturn_t abt1_intr(int irq, void *dev_id)
{
	errind_start(ERRIND_REMOTE);
	printk(KERN_ERR "ERROR: REMOTE Exception detected.\n");
	/* single shot */
	disable_irq(irq);
	return IRQ_HANDLED;
}

void __init errind_remote_init(void)
{
	if (cxd41xx_smode < 0)
		return;
	if (!cxd41xx_smode) {
		if (cxd41xx_abt1_irq < 0)
			return;
		if (request_irq(cxd41xx_abt1_irq, abt1_intr, IRQF_DISABLED,
				ABT1_NAME, &abt1_info)) {
			printk(KERN_ERR "ABT1:ERROR:request_irq %d failed.\n",
			       cxd41xx_abt1_irq);
		}
	}
}

void __exit errind_remote_exit(void)
{
}
