/*
 * arch/arm/plat-cxd41xx/include/mach/pm.h
 *
 * Copyright 2009,2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#ifndef __PLAT_CXD41XX_PM_H
#define __PLAT_CXD41XX_PM_H

#include <mach/pm_export.h>

/* mode of cxd4115_cpu_suspend() */
#define PM_ENTER_NORMAL   0
#define PM_ENTER_ERROR    1
#define PM_ENTER_RESET    2

#ifndef __ASSEMBLY__
#include <asm/cacheflush.h>

static inline void pm_save_virt(u8 *src, u8 *dst, u32 size)
{
	int i;
	for(i = 0; i < size; i++){
		if(!i || !(((unsigned long)src + i) & ~PAGE_MASK)){
			flush_dcache_page(virt_to_page(((unsigned long)src + i) & PAGE_MASK));
			flush_icache_page(current->mm, virt_to_page(((unsigned long)src + i) & PAGE_MASK));
		}
	}

	for(i = 0; i < size; i++){
		dst[i] = src[i];
	}
}

static inline void pm_restore_virt(u8 *src, u8 *dst, u32 size)
{
	int i;
	for(i = 0; i < size; i++){
		dst[i] = src[i];
	}

	for(i = 0; i < size; i++){
		if(!i || !(((unsigned long)dst + i) & ~PAGE_MASK)){
			flush_dcache_page(virt_to_page(((unsigned long)dst + i) & PAGE_MASK));
			flush_icache_page(current->mm, virt_to_page(((unsigned long)dst + i) & PAGE_MASK));
		}
	}
}

extern void cxd41xx_speed_notify(void);
extern void cxd41xx_pm_setup(void);
extern void cxd4115_poweroff(void);
extern unsigned long sleep_phys_sp(void *sp);  /* call from sleep.S */
extern int cxd4115_create_warmbootimage(void); /* call from sleep.S */
extern unsigned int cxd41xx_mem_alive; /* set from sleep.S */
#endif /* !__ASSEMBLY__ */
#endif /* __PLAT_CXD41XX_PM_H */
