#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/udif/types.h>
#include <linux/udif/irq.h>
#include <linux/udif/macros.h>

#define UDIF_DECLARE_INTR(name, i, d) \
UDIF_INTR name = { \
	.irq	= i, \
	.data	= d, \
}

static irqreturn_t udif_handler(int irq, void *dev_id)
{
	UDIF_INTRPT *intr = dev_id;
	UDIF_DECLARE_INTR(hndl, irq, intr->data);
	int ret;

	if (unlikely(!intr)) {
		UDIF_PERR("invalid interrupt irq = %d\n", irq);
		return IRQ_NONE;
	}

	if (unlikely(intr->irq != irq)) {
		UDIF_PERR("%s: Wrong irq = %d, expected irq = %d\n", intr->name, irq, intr->irq);
		return IRQ_NONE;
	}

	ret = intr->hndl(&hndl);
	if (unlikely(ret != UDIF_ERR_OK)) {
		UDIF_PERR("%s: failed handler irq = %d\n", intr->name, irq);
		return IRQ_NONE;
	}

	return IRQ_HANDLED;
}

#define UDIF_IRQF_DEFAULT (IRQF_DISABLED)

UDIF_ERR __udif_request_irq(UDIF_INTRPT *intr, UDIF_INT_CB hndl, UDIF_VP data)
{
	UDIF_ERR ret;

	UDIF_PARM_CHK(!intr, "invalid intr", UDIF_ERR_PAR);
	UDIF_PARM_CHK(!intr->irq, intr->name, UDIF_ERR_PAR);
	UDIF_PARM_CHK(intr->hndl, intr->name, UDIF_ERR_PAR);

	intr->hndl = hndl;
	intr->data = data;

	ret = request_irq(intr->irq, udif_handler, intr->flags | UDIF_IRQF_DEFAULT, intr->name, intr);
	if (unlikely(ret)) {
		UDIF_PERR("failed request_irq(), %s: irq = %d, flags = %08x\n", intr->name, intr->irq, intr->flags | UDIF_IRQF_DEFAULT);
		intr->data = 0;
		intr->hndl = 0;
	}

	return ret;
}

UDIF_ERR __udif_free_irq(UDIF_INTRPT *intr)
{
	UDIF_PARM_CHK(!intr, "invalid intr", UDIF_ERR_PAR);
	UDIF_PARM_CHK(!intr->irq, intr->name, UDIF_ERR_PAR);
	UDIF_PARM_CHK(!intr->hndl, intr->name, UDIF_ERR_PAR);

	free_irq(intr->irq, intr);

	intr->data = 0;
	intr->hndl = 0;

	return UDIF_ERR_OK;
}

EXPORT_SYMBOL(__udif_request_irq);
EXPORT_SYMBOL(__udif_free_irq);
